#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <unistd.h>
#include <string>
#include <iostream>
//#include "simplex_downhill.h"

#define N 540
#define DIM 7
#define MAXNODE 5000
#define LANDMARK 16
#define MAXCLUSTER 500

#define BETA 16

// First 10 base layer nodes
#define BINNINGHOST 16
#define BINNING_LENGTH 1

#define MAX_ERR 1e20

#define Rand() ((double)rand())/RAND_MAX

int input_node_number;
int order[MAXNODE];
int marked[MAXNODE];
double matrix[MAXNODE][MAXNODE];
int nb_count[MAXNODE];
double coordinate[MAXNODE][DIM];
int landmark_nodes[MAXNODE][LANDMARK];
int as_number[MAXNODE];
int as_distribute[65536];
char binning[MAXNODE][BINNINGHOST];

int first_beta[BETA];
int first_beta_test[BETA];
int bin_host[BINNINGHOST];
double bin_coordinate[BINNINGHOST][DIM];
int belong[MAXNODE];

int global_cluster[MAXNODE]; // Form cluster by global information~

	

FILE  *inputfile, *outputfile, *logfile;

#define TINY 1.0e-10
#define NMAX 500000

double ftol = 0.00001;

/*
  note: all effective array indices are 1..N, not 0..N-1

  simplex - the initial simplex, must contain d + 1 points
  values - the initial function values at the simplex points
  d - the number of dimensions
  ftol - the tolerate for convergence
  obj - the objective function to minimize
  num_eval - the number of evaluations of obj
  stuff - things that are needed by obj / test_node_number
*/
void simplex_downhill1(double simplex[][DIM*BINNINGHOST+1], double *values, int d, double ftol,
	 	      double (*obj)(double *, int, int), int *num_eval,
		      int stuff) {

  int i, j, low, high, second_high, ssize;
  double rtol, sum, *simplex_sum, *test_point, test_value, mult;

  /* initializations */
  *num_eval = 0;
  ssize = d + 1; /* size of the simplex */

  /* for each dimension, pre-compute the sum of all points */
  /* this is used later to compute the average coordinates */
  simplex_sum = (double*) malloc(sizeof(double)*(d+1)); /* array starts at 1 */
  for (i=1; i<=d; i++) { 
    sum = 0.0;
    for(j=1; j<=ssize; j++) {
      sum = sum + simplex[j][i];
    }
    simplex_sum[i] = sum;
  }

  /* create a test point */
  test_point = (double*) malloc(sizeof(double)*(d+1)); /* array starts at 1 */


  /* begin algorithm */
  while (1) {

    /* find the lowest point, the highest point and the second highest
       point in the simplex */
    if (values[1] > values[2]) {
      low = 2;
      high = 1;
      second_high = 2;
    } else {
      low = 1;
      high = 2;
      second_high = 1;
    }
    for (i=1; i<=ssize; i++) {
      if (values[i] > values[high]) {
		second_high = high;
		high = i;
      } else if (values[i] > values[second_high] && i != high) {
		second_high = i;
      } else if (values[i] <= values[low]) {
		low = i;
      }
    }


    /* we will quit if there are too many tries
       the tolerance is met, or if the function
       value is so low that we really don't care anymore
    */
    rtol=2.0*fabs(values[high]-values[low])/(fabs(values[high])+fabs(values[low])+TINY);
    if (*num_eval >= NMAX || rtol < ftol || values[low] < 1.0e-6) {
      values[1] = values[low];
      for (i=1; i<=d; i++) {
		simplex[1][i] = simplex[low][i];
      }
      break;
    }


    /* first try to REFLECT the high point ACROSS the mean point */
    /* i.e. want (x_mean - x_high) = - (x_mean - x_test) */
	// R = 2 * Mean - Worest
    mult = 2.0/d;
    for (i=1; i<=d; i++) {
      test_point[i] = (simplex_sum[i] - simplex[high][i])*mult - simplex[high][i];
    }

    test_value = (*obj)(&test_point[1], d, stuff);

    (*num_eval)++;
    if (test_value < values[high]) {
      /* better point, update the simplex, update also the sum */
      values[high] = test_value;
      for (i=1; i<=d; i++) {
		simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
		simplex[high][i] = test_point[i];
      }
    }

    if (test_value <= values[low]) {
      /* the new point is even better than our lowest point, okay, now,
	 extend the point we just found even further
	 i.e. want 2*(x_mean - x_high) = (x_mean - x_test)
      */
	  // E = 2R - M, high has been changed
      mult = -1.0/d;
      for (i=1; i<=d; i++) {
		test_point[i] = (simplex_sum[i] - simplex[high][i])*mult + 2.0*simplex[high][i];
      }

      test_value = (*obj)(&test_point[1], d, stuff);

      (*num_eval)++;
      if (test_value < values[high]) {
	    /* better point, update the simplex, update also the sum */
		values[high] = test_value;
		for (i=1; i<=d; i++) {
			simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
			simplex[high][i] = test_point[i];
		}
      }

    } else if (test_value >= values[second_high]) {
      /* the new point is still the highest, no improvement,
         so we are going to shrink the high point towards the mean
	 i.e. want (x_mean - x_high) = 2*(x_mean - x_test)
      */
      mult = 0.5/d;
      for (i=1; i<=d; i++) {
		test_point[i] = (simplex_sum[i] - simplex[high][i])*mult + 0.5*simplex[high][i];
      }

      test_value = (*obj)(&test_point[1], d, stuff);

      (*num_eval)++;
      if (test_value < values[high]) {
		/* better point, update the simplex, update also the sum */
		values[high] = test_value;
		for (i=1; i<=d; i++) {
			simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
			simplex[high][i] = test_point[i];
		}
      } else {
		/* no good, we better just contract the whole simplex
	   toward the low point
		*/
		for (i=1; i<=ssize; i++) {
		if (i != low) {
			for (j=1; j<=d; j++) {
			simplex[i][j]=test_point[j]=0.5*(simplex[i][j]+simplex[low][j]);
			}
			values[i]=(*obj)(&test_point[1], d, stuff);
		}
	}
	*num_eval = *num_eval + d;

	/* recompute the sums */
	for (i=1; i<=d; i++) { 
	  sum = 0.0;
	  for(j=1; j<=ssize; j++) {
	    sum = sum + simplex[j][i];
	  }
	  simplex_sum[i] = sum;
	}
      }
    }
  }

  free(simplex_sum);
  free(test_point);

}

void simplex_downhill2(double simplex[][DIM+1], double *values, int d, double ftol,
	 	      double (*obj)(double *, int, int), int *num_eval,
		      int stuff) {

  int i, j, low, high, second_high, ssize;
  double rtol, sum, *simplex_sum, *test_point, test_value, mult;

  /* initializations */
  *num_eval = 0;
  ssize = d + 1; /* size of the simplex */

  /* for each dimension, pre-compute the sum of all points */
  /* this is used later to compute the average coordinates */
  simplex_sum = (double*) malloc(sizeof(double)*(d+1)); /* array starts at 1 */
  for (i=1; i<=d; i++) { 
    sum = 0.0;
    for(j=1; j<=ssize; j++) {
      sum = sum + simplex[j][i];
    }
    simplex_sum[i] = sum;
  }

  /* create a test point */
  test_point = (double*) malloc(sizeof(double)*(d+1)); /* array starts at 1 */


  /* begin algorithm */
  while (1) {

    /* find the lowest point, the highest point and the second highest
       point in the simplex */
    if (values[1] > values[2]) {
      low = 2;
      high = 1;
      second_high = 2;
    } else {
      low = 1;
      high = 2;
      second_high = 1;
    }
    for (i=1; i<=ssize; i++) {
      if (values[i] > values[high]) {
		second_high = high;
		high = i;
      } else if (values[i] > values[second_high] && i != high) {
		second_high = i;
      } else if (values[i] <= values[low]) {
		low = i;
      }
    }


    /* we will quit if there are too many tries
       the tolerance is met, or if the function
       value is so low that we really don't care anymore
    */
    rtol=2.0*fabs(values[high]-values[low])/(fabs(values[high])+fabs(values[low])+TINY);
    if (*num_eval >= NMAX || rtol < ftol || values[low] < 1.0e-6) {
      values[1] = values[low];
      for (i=1; i<=d; i++) {
		simplex[1][i] = simplex[low][i];
      }
      break;
    }


    /* first try to REFLECT the high point ACROSS the mean point */
    /* i.e. want (x_mean - x_high) = - (x_mean - x_test) */
	// R = 2 * Mean - Worest
    mult = 2.0/d;
    for (i=1; i<=d; i++) {
      test_point[i] = (simplex_sum[i] - simplex[high][i])*mult - simplex[high][i];
    }

    test_value = (*obj)(&test_point[1], d, stuff);

    (*num_eval)++;
    if (test_value < values[high]) {
      /* better point, update the simplex, update also the sum */
      values[high] = test_value;
      for (i=1; i<=d; i++) {
		simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
		simplex[high][i] = test_point[i];
      }
    }

    if (test_value <= values[low]) {
      /* the new point is even better than our lowest point, okay, now,
	 extend the point we just found even further
	 i.e. want 2*(x_mean - x_high) = (x_mean - x_test)
      */
	  // E = 2R - M, high has been changed
      mult = -1.0/d;
      for (i=1; i<=d; i++) {
		test_point[i] = (simplex_sum[i] - simplex[high][i])*mult + 2.0*simplex[high][i];
      }

      test_value = (*obj)(&test_point[1], d, stuff);

      (*num_eval)++;
      if (test_value < values[high]) {
	    /* better point, update the simplex, update also the sum */
		values[high] = test_value;
		for (i=1; i<=d; i++) {
			simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
			simplex[high][i] = test_point[i];
		}
      }

    } else if (test_value >= values[second_high]) {
      /* the new point is still the highest, no improvement,
         so we are going to shrink the high point towards the mean
	 i.e. want (x_mean - x_high) = 2*(x_mean - x_test)
      */
      mult = 0.5/d;
      for (i=1; i<=d; i++) {
		test_point[i] = (simplex_sum[i] - simplex[high][i])*mult + 0.5*simplex[high][i];
      }

      test_value = (*obj)(&test_point[1], d, stuff);

      (*num_eval)++;
      if (test_value < values[high]) {
		/* better point, update the simplex, update also the sum */
		values[high] = test_value;
		for (i=1; i<=d; i++) {
			simplex_sum[i] = simplex_sum[i] - simplex[high][i] + test_point[i];
			simplex[high][i] = test_point[i];
		}
      } else {
		/* no good, we better just contract the whole simplex
	   toward the low point
		*/
		for (i=1; i<=ssize; i++) {
		if (i != low) {
			for (j=1; j<=d; j++) {
			simplex[i][j]=test_point[j]=0.5*(simplex[i][j]+simplex[low][j]);
			}
			values[i]=(*obj)(&test_point[1], d, stuff);
		}
	}
	*num_eval = *num_eval + d;

	/* recompute the sums */
	for (i=1; i<=d; i++) { 
	  sum = 0.0;
	  for(j=1; j<=ssize; j++) {
	    sum = sum + simplex[j][i];
	  }
	  simplex_sum[i] = sum;
	}
      }
    }
  }

  free(simplex_sum);
  free(test_point);

}

double linear_dist(double *a, double *b) {
	 int i;
	 double dist;

	 int norm = 2;
	 dist = 0;

	for (i=0; i<DIM; i++)
  	{
		if (norm == 2) {
			dist += (a[i] - b[i])*(a[i] - b[i]);
    		} else {
      			dist += pow(fabs(a[i] - b[i]), norm);
    		}
  	}

	if (norm == 1) {
		return dist;
	} else
 		return pow(dist, 1.0/norm);
}

double fit_p1(double *array, int num, int backup) // Layer 0 Global GNP
{
	double dist, sum, error, actual;
	int i, j, temp;

	sum = 0;
	for(i=0;i<BINNINGHOST;i++)
		for(j=0;j<BINNINGHOST;j++)
		{
			if (i == j) continue;
			actual = matrix[bin_host[i]][bin_host[j]];
			if (actual < 0) {
				//printf("Fit_p1: No route to the host!"); 
				continue;}
			dist = linear_dist(&array[i*DIM], &array[j*DIM]);
//			error = (dist-actual)*(dist-actual);
			error = ((dist - actual)/actual)*((dist - actual)/actual);	 // one bug? directly sqrt...
			sum += error;
//			printf("<%.1f,%.1f>", dist, actual);
		}
	return sqrt(sum);
}

/* to fit a target in the model of mp model probes
   stuff: array of distaces between target and probes
 */
 double fit_p2(double *array, int num, int test_node)
{
	double dist, sum, error, actual;
	int i, j, temp;

	sum = 0;
	for(i=0;i<LANDMARK;i++)
	{
		temp = landmark_nodes[test_node][i];
		actual = matrix[test_node][temp];
		if (actual < 0) {printf("Fit_p2: No route to the host!"); continue;}
		dist = linear_dist(&array[0], &coordinate[temp][0]);
//		printf("<%d,%d>", test_node, temp);
//		printf("[%lf,%lf]", actual, dist);
		error = ((dist - actual)/actual)*((dist - actual)/actual);	 // one bug? directly sqrt...
		//error = (dist-actual)*(dist-actual);
    		sum += error;
 	}
	return sqrt(sum);
}

 double fit_p3(double *array, int num, int test_node)
{
	double dist, sum, error, actual;
	int i, j, temp;

	sum = 0;
	for(i=0;i<BINNINGHOST;i++)
	{
		temp = bin_host[i];
		if (temp == test_node)
		{
			printf("Error!!");
			exit(1);
		}
		actual = matrix[test_node][temp];
		if (actual < 0) {//printf("Fit_p3: No route to the host!"); 
		continue;}
//		dist = linear_dist(&array[0], &bin_coordinate[i][0]);
		dist = linear_dist(&array[0], &coordinate[temp][0]);


//		printf("<%d,%d>", test_node, temp);
//		printf("[%lf,%lf]", actual, dist);
		error = ((dist - actual)/actual)*((dist - actual)/actual);
		//error = (dist-actual)*(dist-actual);
		sum += error;
 	}
	return sqrt(sum);
}



// Returns a number (a,b]
int unifRand (int a, int b) {
  assert (a >= 0 && b >= 0);
  int diff = abs((a-b));
  int randValue = (int)(Rand()*diff);
  if (randValue == diff)
  {
  	randValue = diff -1;
	printf("o");
  }
  return (int)(a + randValue);
}

double cost_calc() // first_beta[BETA]
{
	int i, j, k, conf;
	int selected[MAXNODE];

	double result = 0.0;
	double min_dist;

	for(i=0;i<input_node_number;i++)
	{
		min_dist = 10000;
		conf = 1;
		for(j=0;j<BETA;j++)
		{
			if (matrix[i][first_beta_test[j]] < 0)
			{
				continue;
			}
			if (i == first_beta[j])
			{
				conf = -1;
				break;
			}
			if (matrix[i][first_beta_test[j]] < min_dist)
			{
				min_dist = matrix[i][first_beta_test[j]];
				conf = j;
			}
		}
		selected[i] = conf;
		if (conf != -1)
			result += min_dist * min_dist;
		
	}
	return result;
}

void Landmark_Hierarchy_Construction()
{
	double gamma = 100;
	double xita = 90;
	int i, j, k;
	int test;
	int ok;
	int round, turn;
	int selected[MAXNODE];
	int from, to;
	double min_dist, max_dist;

	double quality[BETA];
	int num[BETA];

	for(i=0;i<MAXNODE;i++) selected[i] = -1;
	printf("\nForming global cluster...\n");
	i = 0;
	while(1)
	{
		test = rand() % input_node_number;
		ok = 1;
		for(j=0;j<i;j++)
		{
			// Ever Selected?
			if (test == first_beta[j])
			{
				ok = 0;
				break;
			}
			// Distance above gamma?

			if (matrix[test][first_beta[j]] > -0.5 && matrix[test][first_beta[j]] < gamma)
//			if (matrix[test][first_beta[j]] > -0.5)
			{
				ok = 0;
				break;
			}

		}
		if (ok == 0) continue;
		first_beta[i] = test;
		selected[test] = i;
		i++;
		if (i == BETA) break;
	}
	for(i=0;i<BETA;i++)
	{
		printf("%d (%d) ", first_beta[i], nb_count[first_beta[i]]);
		first_beta_test[i] = first_beta[i];
	}
	double now_cost = cost_calc();
	printf("\n");
//	printf("0: %.2f\n", now_cost);
	for(i=0;i<MAXCLUSTER;i++)
	{
		int ok, find_min, belong;
		double min_dist;
		
		k = rand() % input_node_number;
		for(j=0;j<input_node_number;j++)
		{
			// eval : (k+j) % input_node_number
			ok = 1;
			for(find_min=0;find_min<BETA;find_min++)
			{
				if (first_beta[find_min] == (k+j) % input_node_number)
				{
					ok = 0;
					break;
				}
			}
			if (ok == 0) continue;			
			min_dist = 1000000;
			for(find_min=0;find_min<BETA;find_min++)
			{
				if (matrix[first_beta[find_min]][(k+j) % input_node_number] < 0) continue;
				if (min_dist>matrix[first_beta[find_min]][(k+j) % input_node_number])
				{
					min_dist = matrix[first_beta[find_min]][(k+j) % input_node_number];
					belong = find_min;
				}
			}
			for(find_min=0;find_min<BETA;find_min++)				
			{
				if (find_min == belong)
					first_beta_test[find_min] = j;
				else
					first_beta_test[find_min] = first_beta[find_min];
			}
			if (cost_calc()<now_cost)
			{
				now_cost = cost_calc();
				break;
			}
		}
		if (j == input_node_number)
		{
			printf("Best Result!\n");
			break;
		}
		for(j=0;j<BETA;j++) first_beta[j] = first_beta_test[j];
		printf("%d: %.2f\n", i+1, now_cost);
	}
	for(i=0;i<BETA;i++) bin_host[i] = first_beta[i];

	printf("Binning: ");
	for(i=0;i<BINNINGHOST;i++)
		printf("%d ", bin_host[i]);
	printf("Cost: %.2f", now_cost);
//	exit(1);
		
}

void Shuffle()
{

	int i, j, k;
	int x, y;
	int temp;
	int count;
	for(i=0;i<input_node_number;i++) order[i] = i;
	// Shuffle
	for(i=0;i<input_node_number;i++)
	{
		x = i;
		y = unifRand(0, input_node_number);
		// Exchange x and y
//		cout<<x<<y<<endl;
//		printf("%d %d\n", x, y);
		temp = order[x]; order[x] = order[y]; order[y] = temp;
	}
	printf("Selected Nodes:\n");
	for(i=0;i<input_node_number;i++) marked[i] = -1;
	count = 0;
	for(i=0;i<input_node_number;i++)
	{
		if (nb_count[order[i]] > 8)
		{
			printf("%d ", order[i]);
			marked[order[i]] = count;
			count++;
			if (count >= N)
				break;
		}
	
	}
/*
	for(i=0;i<N;i++)
	{
		printf("%d ", order[i]);
		marked[order[i]] = i;
	}
	*/
/*	
	printf("\nSelected Nodes (CERNET & Internet2) 45nodes:\n");
	j = 0;
	for(i=0;i<input_node_number;i++)
	{
		marked[i] = -1;
		if (as_number[i] == 4538 || as_number[i] == 11537)
		{
			marked[i] = j;
			j++;
		}
	}
	*/
}

int already_landmark(int test_node, int target)
{
	int i;
	for(i=0;i<LANDMARK;i++)
	{
		if (landmark_nodes[test_node][i] == target)
			return 1;
	}
	return -1;
}
int already_cluster(int test_node, int target)
{
	int i, temp = 0;
	if (belong[target] == belong[test_node])
	{
		for(i=0;i<LANDMARK;i++)
		{
			if (landmark_nodes[test_node][i] == -1)
				break;
			if (belong[landmark_nodes[test_node][i]] == belong[target])
				temp++;			
		}
		if (temp >= 2) return 1;
		return -1;
	}
	for(i=0;i<LANDMARK;i++)
	{
		if (landmark_nodes[test_node][i] == -1)
			break;
		if (belong[landmark_nodes[test_node][i]] == belong[target])
			return 1;
	}
	return -1;
}


//simplex_downhill(p,y,num,localftol,fit_func,&j,d);

void FindBin()
{
	int i, j, k;
	int i1, i2;
	double distance[BINNINGHOST];
//	char binning[N][BINNINGHOST];
	int temp_i;
	double temp_d;

	printf("\n");
	for(i=0;i<input_node_number;i++)
	{
//		printf("<%d,", i);
		i1 = i;
		// From the ith base layer node to the jth binning node
		for(j=0;j<BINNINGHOST;j++)
		{
			binning[i][j] = j;
			i2 = bin_host[j];
/*
			for(k=0;k<input_node_number;k++) 
				if (marked[k] == j)
				{
					i2 = k;
					break;
				}
*/
			if (i1 != i2)
				distance[j] = matrix[i1][i2];
			else
			{
				distance[j] = 0;
				printf("@");
			}
			if (distance[j] < -0.5)
				distance[j] = 1500;
//			printf("%f ", distance[j]);
		}
		for(j=0;j<BINNINGHOST-1;j++)
			for(k=j+1;k<BINNINGHOST;k++)
			{
				if (distance[j] > distance[k])
				{
					temp_d = distance[j]; distance[j] = distance[k]; distance[k] = temp_d;
					temp_i = binning[i][j]; binning[i][j] = binning[i][k]; binning[i][k] = temp_i;
				}
			}
//		for(j=0;j<BINNING_LENGTH;j++)
//			printf("%d>", binning[i][j]);
//		printf(" ");
	}

	// Cluster Analysis
	printf("Clustering\n");
	for(i=0;i<N;i++) belong[i] = -1;
	int number = 0;
	int ok;
	int count;
	while(1)
	{
		for(i=0;i<input_node_number;i++)
			if (belong[i] == -1) break;
		if (i == input_node_number)
			break;
		belong[i] = number;
		count = 1;
		// Search
		for(j=i+1;j<N;j++)
		{
			ok = 1;
			for(k=0;k<BINNING_LENGTH;k++)
				if (binning[i][k] != binning[j][k])
				{
					ok = 0;
					break;
				}
			if (ok == 1)
			{
				belong[j] = number;
				count++;
			}			

		}
		printf("%d ", count);
		number++;
	}
	printf("\nTotal %d Cluster\n", number);
}

void Pure_GNP(FILE *fp3)
{
	int i, j, k;
	int total_try1, total_try2;
	int num = DIM;		
	double localftol = ftol;
	int num_eval;
	int stuff;
	int a, b;

	int ok;
	
	double min_fit = 100000000;

	printf("Pure GNP\n");
	
	// Create Links for extended layer nodes

	for(j=0;j<BINNINGHOST;j++)
	{
		for(k=0;k<DIM;k++)
		{
				coordinate[bin_host[j]][k] = bin_coordinate[j][k];
		}
	}
	for(i=0;i<input_node_number;i++)
	{
		ok = 1;
		for(j=0;j<BINNINGHOST;j++)
		{
			if (bin_host[j] == i)
			{
				ok = 0;
				break;
			}
		}


		printf("%d: <%d>\n", i, nb_count[i]);
		if (ok == 0) continue;	

		printf("\n");
		// Process GNP to the output file
		// GNPing............
	
		double p[DIM+1+1][DIM+1];
		double y[DIM+1];
	
		num = DIM;		
		localftol = ftol;
			
		min_fit = 10000;
	
		for(total_try1 = 0; total_try1 < 3; total_try1++)
		{
			for(total_try2 = 0; total_try2 < 3; total_try2++)
			{
				if (total_try2 == 0)
				{

					a = 1;
					for(b=1;b<=DIM;b++)
					{	
						p[a][b] = (double)(rand() % 500 - 250); 				
					}				
					y[a] = fit_p3(&p[a][1], DIM, i); printf("%.2f ", y[a]);
	
					for(a=2;a<=DIM+1;a++)
					{
						for(b=1;b<=DIM;b++)
						{	
							p[a][b] = p[1][b];	
						}				
						p[a][a-1] += 2000;
						y[a] = fit_p3(&p[a][1], DIM, i); printf("%.2f ", y[a]);
					}
				}
				else
				{
					y[1] = fit_p3(&p[1][1], DIM, i); printf("%.2f ", y[1]);
					for(a=2;a<=DIM+1;a++)
					{
						
						for(b=1;b<=DIM;b++)
						{	
							p[a][b] = p[1][b];					
						}
						
						p[a][a-1] += 2000;
					
						y[a] = fit_p3(&p[a][1], DIM, i); printf("%.2f ", y[a]);
					}
				}
				simplex_downhill2(p,y,num,localftol,fit_p3,&j,i);
				printf("Fit: %.2f J : %d\n", fit_p3(&p[1][1], DIM, i), j);
				if (fit_p3(&p[1][1], DIM, i) < min_fit)
				{
					min_fit = fit_p3(&p[1][1], DIM, i);
					for(j=0;j<DIM;j++)
						coordinate[i][j] = p[1][j+1];
				}
			}
		}
	}	
	printf("All Coordinates:\n");
	FILE *tempfp;
	tempfp = fopen("gnp16_coord.txt", "w");
	for(i=0;i<input_node_number;i++)
	{
		printf("Node %d: ", i);
		for(j=0;j<DIM;j++) 
		{
			printf("%lf ", coordinate[i][j]);
			fprintf(tempfp, "%lf ", coordinate[i][j]);			
		}
		printf("\n");
		fprintf(tempfp, "\n");
	}
	fclose(tempfp);

	printf("RE:\n");
	printf("Saving...");
	for(i=0;i<input_node_number;i++)
		for(j=0;j<input_node_number;j++)
		{
			if (matrix[i][j] == -1)
				continue;
//			0 1 re 0.156878 d 154.194443 l 133.284988 
//			printf("%d %d re ", i, j);
			double dist, actual;
			actual = matrix[i][j];
			dist = linear_dist(coordinate[i], coordinate[j]);
			if (dist < actual)
			{
//				printf("%lf ", fabs(actual-dist)/dist);
				fprintf(fp3, "%lf ", fabs(actual-dist)/dist);
			}
			else
			{
//				printf("%lf ", fabs(actual-dist)/actual);
				fprintf(fp3, "%lf ", fabs(actual-dist)/actual);
			}
//			printf("d %lf l %lf", dist, actual);
//			printf("\n");
		}
}

void Process(FILE *fp1, FILE *fp2, FILE *fp3)
{
	FILE *baselayer;
	int myId, yourId;
	double rawLatencySample, tempcoord;
	int i, j, k, tempnum;
	int a, b;
	for(i=0;i<input_node_number;i++) nb_count[i] = 0;
	for(i=0;i<input_node_number;i++)
		for(j=0;j<input_node_number;j++)
			matrix[i][j] = -1;
 	while (fscanf (fp1, "%d %d %lf\n", &myId, &yourId, &rawLatencySample) > 0) {   
//		printf("%d %d %lf\n", myId, yourId, rawLatencySample);
		matrix[myId][yourId] = rawLatencySample;
		nb_count[myId]++;
		matrix[yourId][myId] = rawLatencySample;
		nb_count[yourId]++;

/*
		if (marked[myId] != -1 && marked[yourId] != -1)
		{
			printf("%d [%d] %d [%d] %lf\n", myId, marked[myId], yourId, marked[yourId], rawLatencySample);
			fprintf(fp2, "%d %d %lf\n",  marked[myId], marked[yourId], rawLatencySample);		
		}
*/

	}
	fclose(fp1);

//	printf("Neighbor Count: ");
//	for(i=0;i<input_node_number;i++) printf("%d ", nb_count[i]);

	Landmark_Hierarchy_Construction();


	printf("\n");
	Shuffle();
	FindBin();
	for(i=0;i<input_node_number;i++)
		for(j=0;j<input_node_number;j++)
		{
			if (matrix[i][j] > 0 && marked[i] != -1 && marked[j] != -1)
			{
				fprintf(fp2, "%d %d %lf\n",  marked[i], marked[j], matrix[i][j]);		
			}
		}


/*
	system("./vivaldi -n 540 -x 7 -k 16 -l fourty.txt -O tt -g cr -r 200000");
	baselayer = fopen("tt.coord", "r");
	for(i=0;i<N;i++)
	{	
		fscanf(baselayer, "%d", &tempnum);
		// search the right k
		for(k=0;k<input_node_number;k++)
			if (marked[k] == i)
				break;
		printf("%d %d : ", i, k);
		for(j=0;j<DIM;j++)
		{
			fscanf(baselayer, "%lf", &tempcoord);
			coordinate[k][j] = tempcoord;
			printf("%.2f ", tempcoord);			
		}
		printf("\n");
	}
	fclose(baselayer);
*/

	// Layer 0 GNPing
	int total_try1, total_try2;
	int num = DIM * BINNINGHOST;		
	double localftol = ftol;
	int num_eval;
	int stuff;
	
	double min_fit = 100000000;
	double p0[DIM*BINNINGHOST+1+1][DIM*BINNINGHOST+1];
	double y0[DIM*BINNINGHOST+1];
	
	for(total_try1 = 0; total_try1 < 6; total_try1++)
	{
		for(total_try2 = 0; total_try2 < 4; total_try2++)
		{
			if (total_try2 == 0)
			{
				a = 1;
				for(b=1;b<=DIM*BINNINGHOST;b++)
				{	
					p0[a][b] = (double)(rand() % 500 - 250);				
//					printf("%.2f ", p0[a][b]);
				}				
				y0[a] = fit_p1(&p0[a][1], DIM*BINNINGHOST, 0); printf("%.2f ", y0[a]);

				for(a=2;a<=DIM*BINNINGHOST+1;a++)
				{
					for(b=1;b<=DIM*BINNINGHOST;b++)
					{	
						p0[a][b] = p0[1][b];				
					}				
					p0[a][a-1] += 2000;

					y0[a] = fit_p1(&p0[a][1], DIM*BINNINGHOST, 0); printf("%.2f ", y0[a]);
				}
			}
			else
			{
				y0[1] = fit_p1(&p0[1][1], DIM*BINNINGHOST, 0); printf("%.2f ", y0[1]);
				for(a=2;a<=DIM*BINNINGHOST+1;a++)
				{							
					for(b=1;b<=DIM*BINNINGHOST;b++)
					{	
						p0[a][b] = p0[1][b];					
					}
						
					p0[a][a-1] += 2000; 					
					y0[a] = fit_p1(&p0[a][1], DIM*BINNINGHOST, 0); printf("%.2f ", y0[a]);
				}
			}
			simplex_downhill1(p0,y0,num,localftol,fit_p1,&j,i);
			printf("Fit: %.2f J : %d\n", fit_p1(&p0[1][1], DIM*BINNINGHOST, 0), j);
			if (fit_p1(&p0[1][1], DIM*BINNINGHOST, 0) < min_fit)
			{
				min_fit = fit_p1(&p0[1][1], DIM*BINNINGHOST, 0);
				for(i=0;i<BINNINGHOST;i++)
				{
					for(j=0;j<DIM;j++)
					{
						bin_coordinate[i][j] = p0[1][i*DIM+j+1];
					}
				}
			}
		}
	}
	printf("All Layer 0 Coordinates:\n");
	//	bin_coordinate[BINNINGHOST][DIM];
	for(i=0;i<BINNINGHOST;i++)
	{
		for(j=0;j<DIM;j++)
		{
			printf("%.2f ", bin_coordinate[i][j]);
		}
		printf("\n");
	}
	Pure_GNP(fp3);
	exit(1);
	
	// Finish Layer 0 GNPing


	printf("Extended Layer\n");

	// Create Links for extended layer nodes
	for(i=0;i<input_node_number;i++)
	{
		for(j=0;j<LANDMARK;j++) landmark_nodes[i][j] = -1;
		if (marked[i] != -1) // In base layer
			continue;

		printf("%d: <%d>\n", i, nb_count[i]);

		// LANDMARK Selection
//		int j1 = 0;

		for(j=0;j<LANDMARK;j++)
		{
			int target = i;
			while (target == i || matrix[i][target] <= 0 || 
				marked[target] == -1 || already_landmark(i, target) == 1)
//			while (target == i || matrix[i][target] <= 0 || 
//				marked[target] == -1 || already_landmark(i, target) == 1 || already_cluster(i, target) == 1)
			{
				target = rand() % input_node_number;
			}
			landmark_nodes[i][j] = target;
//			printf("%d ", target);
		}		

		printf("\n");
		// Process GNP to the output file
		// GNPing............

		double p[DIM+1+1][DIM+1];
		double y[DIM+1];

		num = DIM;		
		localftol = ftol;
		
		min_fit = 10000;

		for(total_try1 = 0; total_try1 < 3; total_try1++)
		{
/*
			for(b=1;b<=DIM;b++) p[1][b] = (double)(rand() % 500 - 250);
			y[1] = fit_p2(&p[1][1], DIM, i);
			for(a=2;a<=DIM+1;a++)
			{
				for(b=1;b<=DIM;b++)
				{	
					p[a][b] = p[1][b];					
				}
				p[a][a-1] += 1000 + rand() % 2000;

				y[a] = fit_p2(&p[a][1], DIM, i); printf("%lf ", y[a]);
			}
			*/
			for(total_try2 = 0; total_try2 < 3; total_try2++)
			{
				if (total_try2 == 0)
				{
					for(a=1;a<=DIM+1;a++)
					{
						for(b=1;b<=DIM;b++)
						{	
							p[a][b] = (double)(rand() % 500 - 250);					
						}				
						if (a>1) p[a][a-1] += 2000;
						y[a] = fit_p2(&p[a][1], DIM, i); printf("%.2f ", y[a]);
					}
/*
					for(b=1;b<=DIM;b++) p[1][b] = (double)(rand() % 500 - 250);					
					y[1] = fit_p2(&p[1][1], DIM, i);  
*/
				}
				else
				{
				y[1] = fit_p2(&p[1][1], DIM, i); printf("%.2f ", y[1]);
				for(a=2;a<=DIM+1;a++)
				{
					/*
					for(b=1;b<=DIM;b++)
					{	
						p[a][b] = p[1][b];					
					}
					*/
					p[a][a-1] += 2000;
				
					y[a] = fit_p2(&p[a][1], DIM, i); printf("%.2f ", y[a]);
				}
				}
				simplex_downhill2(p,y,num,localftol,fit_p2,&j,i);
				printf("Fit: %.2f J : %d\n", fit_p2(&p[1][1], DIM, i), j);
				if (fit_p2(&p[1][1], DIM, i) < min_fit)
				{
					min_fit = fit_p2(&p[1][1], DIM, i);
					for(j=0;j<DIM;j++)
						coordinate[i][j] = p[1][j+1];
				}
			}
		}
	}	
	printf("All Coordinates:\n");
//	FILE *tempfp;
//	tempfp = fopen("gnp_nc16.txt", "w");
	for(i=0;i<input_node_number;i++)
	{
//		printf("Node %d: ", i);
		for(j=0;j<DIM;j++) 
		{
			printf("%lf ", coordinate[i][j]);
//			fprintf(tempfp, "%lf ", coordinate[i][j]);			
		}
		printf("\n");
//		fprintf(tempfp, "\n");
	}
//	fclose(tempfp);
	printf("RE:\n");
	printf("Saving results!\n");
	for(i=0;i<input_node_number;i++)
		for(j=0;j<input_node_number;j++)
		{
			if (matrix[i][j] == -1)
				continue;
//			0 1 re 0.156878 d 154.194443 l 133.284988 
//			printf("%d %d re ", i, j);
			double dist, actual;
			actual = matrix[i][j];
			dist = linear_dist(coordinate[i], coordinate[j]);
			if (dist < actual)
			{
//				printf("%lf ", fabs(actual-dist)/dist);
				fprintf(fp3, "%lf ", fabs(actual-dist)/dist);
			}
			else
			{
//				printf("%lf ", fabs(actual-dist)/actual);
				fprintf(fp3, "%lf ", fabs(actual-dist)/actual);
			}
//			printf("d %lf l %lf", dist, actual);
//			printf("\n");
		}

}


void printUsage()
{
	printf("./pickup InputFile InputNodeNumber OutputFile LogFile\n");
	printf("[e.g.] ./pickup median 226 fourty.txt nc16.txt\n");
}

void process_AS()
{
	FILE *fp;
	int count = 0;
	int i;
	char temp[255];
//	char temp_num[255];
	for(i=0;i<65536;i++) as_distribute[i] = 0;
	// Blog
//	fp = fopen("blog_as.txt", "r");
	// PlanetLab	
	fp = fopen("planetlab_as.txt", "r");
	while(count < input_node_number)
	{
		fgets(temp, 255, fp); // 1st line
//		printf("%s", temp);
		fgets(temp, 255, fp); // 2nd line
//		printf("%s", temp);

		// Get AS number
		int number = 0;
		int point = 0;
//		atoi()
		while(1)
		{
			if (temp[point] < '0' || temp[point] > '9')
			{
				temp[point] = 0;
				break;		
			}
			point++;
		}
//		printf("%d ", atoi(temp));
		as_number[count] = atoi(temp);
		as_distribute[atoi(temp)]++;
		///////////////////////
	
		count++;
	}


	// Statistical
	for(i=0;i<65536;i++)
		if (as_distribute[i] > 0)
			printf("[AS%d,%d]  ", i, as_distribute[i]);
//			printf("[%d],", as_distribute[i]);
//	exit(1);
}

int main (int argc, char **argv) 
{

	int seed = getpid();
	srand(seed);

	if (argc != 5)
	{
		printUsage();
		exit(0);
	}
	int i;


	input_node_number = atoi(argv[2]);
	printf("Node Number : %d %lf\n", input_node_number, Rand());

	// 226 PlanetLab Data Set
//	process_AS();

//	Shuffle();
	inputfile = fopen(argv[1], "r");
	outputfile = fopen(argv[3], "w");
	logfile = fopen(argv[4], "w");
	Process(inputfile, outputfile, logfile);
	fclose(inputfile);
	fclose(outputfile);
	fclose(logfile);
	return 0;	
}

